"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var apollo_cache_1 = require("apollo-cache");
var apollo_cache_inmemory_1 = require("apollo-cache-inmemory");
var constants_1 = require("./constants");
var apollo_utilities_1 = require("apollo-utilities");
var defaultConfig = {
    fragmentMatcher: new apollo_cache_inmemory_1.HeuristicFragmentMatcher(),
    dataIdFromObject: apollo_cache_inmemory_1.defaultDataIdFromObject,
    addTypename: true,
    storeFactory: apollo_cache_inmemory_1.defaultNormalizedCacheFactory,
};
var ReduxCache = (function (_super) {
    __extends(ReduxCache, _super);
    function ReduxCache(config, store) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this) || this;
        _this.optimistic = [];
        _this.watches = [];
        _this.silenceBroadcast = false;
        _this.config = __assign({}, defaultConfig, config);
        _this.store = store;
        _this.addTypename = _this.config.addTypename ? true : false;
        return _this;
    }
    ReduxCache.prototype.restore = function (data) {
        this.store.dispatch({
            type: constants_1.APOLLO_RESTORE,
            data: data
        });
        return this;
    };
    ReduxCache.prototype.extract = function (optimistic) {
        if (optimistic === void 0) { optimistic = false; }
        if (optimistic && this.optimistic.length > 0) {
            var patches = this.optimistic.map(function (opt) { return opt.data; });
            return Object.assign.apply(Object, [this.getReducer()].concat(patches));
        }
        return this.getReducer();
    };
    ReduxCache.prototype.read = function (query) {
        if (query.rootId && this.getReducer()[query.rootId] === undefined) {
            return null;
        }
        var options = {
            store: this.config.storeFactory(this.extract(query.optimistic)),
            query: this.transformDocument(query.query),
            variables: query.variables,
            rootId: query.rootId,
            fragmentMatcherFunction: this.config.fragmentMatcher.match,
            previousResult: query.previousResult,
            config: this.config,
        };
        return apollo_cache_inmemory_1.readQueryFromStore(options);
    };
    ReduxCache.prototype.write = function (write) {
        var data = this.config.storeFactory();
        apollo_cache_inmemory_1.writeResultToStore({
            dataId: write.dataId,
            result: write.result,
            variables: write.variables,
            document: this.transformDocument(write.query),
            store: data,
            dataIdFromObject: this.config.dataIdFromObject,
            fragmentMatcherFunction: this.config.fragmentMatcher.match,
        });
        this.store.dispatch({
            type: constants_1.APOLLO_STORE_WRITE,
            data: data.toObject()
        });
        this.broadcastWatches();
    };
    ReduxCache.prototype.diff = function (query) {
        return apollo_cache_inmemory_1.diffQueryAgainstStore({
            store: this.config.storeFactory(this.extract(query.optimistic)),
            query: this.transformDocument(query.query),
            variables: query.variables,
            returnPartialData: query.returnPartialData,
            previousResult: query.previousResult,
            fragmentMatcherFunction: this.config.fragmentMatcher.match,
            config: this.config,
        });
    };
    ReduxCache.prototype.watch = function (watch) {
        var _this = this;
        this.watches.push(watch);
        return function () {
            _this.watches = _this.watches.filter(function (c) { return c !== watch; });
        };
    };
    ReduxCache.prototype.evict = function (query) {
        throw new Error("evict() is not implemented on Redux Cache");
    };
    ReduxCache.prototype.reset = function () {
        this.store.dispatch({
            type: constants_1.APOLLO_RESET,
        });
        this.broadcastWatches();
        return Promise.resolve();
    };
    ReduxCache.prototype.removeOptimistic = function (id) {
        var _this = this;
        var toPerform = this.optimistic.filter(function (item) { return item.id !== id; });
        this.optimistic = [];
        toPerform.forEach(function (change) {
            _this.recordOptimisticTransaction(change.transaction, change.id);
        });
        this.broadcastWatches();
    };
    ReduxCache.prototype.performTransaction = function (transaction) {
        var alreadySilenced = this.silenceBroadcast;
        this.silenceBroadcast = true;
        transaction(this);
        if (!alreadySilenced) {
            this.silenceBroadcast = false;
        }
        this.broadcastWatches();
    };
    ReduxCache.prototype.recordOptimisticTransaction = function (transaction, id) {
        throw new Error("recordOptimisticTransaction() is not implemented on Redux Cache");
    };
    ReduxCache.prototype.readQuery = function (options, optimistic) {
        if (optimistic === void 0) { optimistic = false; }
        return this.read({
            query: options.query,
            variables: options.variables,
            optimistic: optimistic,
        });
    };
    ReduxCache.prototype.readFragment = function (options, optimistic) {
        if (optimistic === void 0) { optimistic = false; }
        return this.read({
            query: this.transformDocument(apollo_utilities_1.getFragmentQueryDocument(options.fragment, options.fragmentName)),
            variables: options.variables,
            rootId: options.id,
            optimistic: optimistic,
        });
    };
    ReduxCache.prototype.writeQuery = function (options) {
        this.write({
            dataId: 'ROOT_QUERY',
            result: options.data,
            query: this.transformDocument(options.query),
            variables: options.variables,
        });
    };
    ReduxCache.prototype.writeFragment = function (options) {
        this.write({
            dataId: options.id,
            result: options.data,
            query: this.transformDocument(apollo_utilities_1.getFragmentQueryDocument(options.fragment, options.fragmentName)),
            variables: options.variables,
        });
    };
    ReduxCache.prototype.broadcastWatches = function () {
        var _this = this;
        if (this.silenceBroadcast)
            return;
        this.watches.forEach(function (c) {
            var newData = _this.diff({
                query: c.query,
                variables: c.variables,
                previousResult: c.previousResult && c.previousResult(),
                optimistic: c.optimistic,
            });
            c.callback(newData);
        });
    };
    ReduxCache.prototype.getReducer = function () {
        return this.store.getState().apollo;
    };
    ReduxCache.prototype.transformDocument = function (document) {
        if (this.addTypename)
            return apollo_utilities_1.addTypenameToDocument(document);
        return document;
    };
    return ReduxCache;
}(apollo_cache_1.ApolloCache));
exports.ReduxCache = ReduxCache;
//# sourceMappingURL=reduxCache.js.map